/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.client.impl;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.registry.client.api.PetalsRegistryClient;
import org.ow2.petals.registry.client.api.TopologyService;
import org.ow2.petals.registry.client.api.exception.AuthenticationException;
import org.ow2.petals.registry.client.api.exception.ConnectionErrorException;
import org.ow2.petals.registry_overlay.api.Constants;

import com.hazelcast.client.HazelcastClient;
import com.hazelcast.client.config.ClientConfig;
import com.hazelcast.client.config.ClientNetworkConfig;
import com.hazelcast.config.NearCacheConfig;
import com.hazelcast.core.HazelcastInstance;

/**
 * The real Petals Registry Client
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsRegistryClientImpl implements PetalsRegistryClient {

    private final Logger log = Logger.getLogger(PetalsRegistryClientImpl.class.getName());

    private final HazelcastInstance hzlClient;

    private InetAddress host = null;

    private int port = 0;

    private String group = null;

    /**
     * @param host
     *            A hostname where the remote Petals Registry cluster runs. If set to <code>null</code> or empty, the
     *            host name is <code>localhost</code>.
     * 
     * @param port
     *            The port of the remote Petals Registry cluster. If set to <code>null</code>, the default port
     *            {@link PetalsRegistryClient#DEFAULT_PORT} is used.
     * @param group
     *            Group name of the credentials required to establish a connection to the remote Petals Registry
     *            cluster. If set to <code>null</code>, the default value is used:
     *            {@value Constants#DEFAULT_CREDENTIALS_GROUP}.
     * @param password
     *            Password of the credentials required to establish a connection to the remote Petals Registry cluster.
     *            If set to <code>null</code>, the default value is used:
     *            {@value Constants#DEFAULT_CREDENTIALS_PASSWORD}.
     * @throws AuthenticationException
     *             When credentials are rejected by the Petals Registry cluster
     * @throws ConnectionErrorException
     *             An error occurs establishing the connection to the Petals Registry cluster
     * 
     */
    public PetalsRegistryClientImpl(final String host, final Integer port, final String group, final String password)
            throws ConnectionErrorException {
        // Connect to the Hazelcast cluster
        final ClientConfig cltCfg = new ClientConfig();

        this.group = (group == null ? Constants.DEFAULT_CREDENTIALS_GROUP : group);
        cltCfg.getGroupConfig().setName(this.group);
        cltCfg.getGroupConfig().setPassword(password == null ? Constants.DEFAULT_CREDENTIALS_PASSWORD : password);
        final ClientNetworkConfig networkConfig = new ClientNetworkConfig();
        cltCfg.setNetworkConfig(networkConfig);
        
        try {
            this.host = InetAddress.getByName(host);
        } catch (final UnknownHostException e) {
            throw new ConnectionErrorException(e);
        }
        if (port == null) {
            this.port = Constants.DEFAULT_PORT;
        } else {
            this.port = port.intValue();
        }
        final String address = (host == null || host.isEmpty() ? "localhost" : host) + ":"
                + (port == null ? Constants.DEFAULT_PORT : port.toString());
        if (this.log.isLoggable(Level.CONFIG)) {
            this.log.config("Member: " + address);
        }
        networkConfig.addAddress(address);

        final NearCacheConfig defaultNearCacheConfig = new NearCacheConfig();
        defaultNearCacheConfig.setName("*");
        cltCfg.addNearCacheConfig(defaultNearCacheConfig);

        // We use the current classloader
        cltCfg.setClassLoader(Thread.currentThread().getContextClassLoader());

        try {
            this.hzlClient = HazelcastClient.newHazelcastClient(cltCfg);
        } catch (final RuntimeException e) {
            if (e.getCause() != null && e.getCause() instanceof com.hazelcast.client.AuthenticationException) {
                throw new AuthenticationException();
            } else {
                throw new ConnectionErrorException(e);
            }
        }
    }

    @Override
    public void disconnect() throws ConnectionErrorException {
        this.hzlClient.shutdown();
    }

    @Override
    public InetAddress getHost() {
        return this.host;
    }

    @Override
    public int getPort() {
        return this.port;
    }

    @Override
    public String getGroup() {
        return this.group;
    }

    @Override
    public TopologyService getTopologyService() {
        return new TopologyServiceImpl(this.hzlClient);
    }

}
